package com.vendor.platform.core.internal;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.vendor.platform.core.IToolManager;
import com.vendor.platform.core.Messages;
import com.vendor.platform.core.PlatformCoreActivator;
import com.vendor.platform.core.ToolDescriptor;
import com.vendor.platform.core.internal.P2Utils;
import com.vendor.platform.core.internal.StringConstants;
import com.vendor.platform.core.internal.ToolFilter;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.core.runtime.URIUtil;
import org.eclipse.equinox.internal.p2.director.ProfileChangeRequest;
import org.eclipse.equinox.p2.core.IProvisioningAgent;
import org.eclipse.equinox.p2.core.IProvisioningAgentProvider;
import org.eclipse.equinox.p2.engine.IEngine;
import org.eclipse.equinox.p2.engine.IPhaseSet;
import org.eclipse.equinox.p2.engine.IProfile;
import org.eclipse.equinox.p2.engine.IProfileRegistry;
import org.eclipse.equinox.p2.engine.IProvisioningPlan;
import org.eclipse.equinox.p2.engine.PhaseSetFactory;
import org.eclipse.equinox.p2.engine.ProvisioningContext;
import org.eclipse.equinox.p2.metadata.IInstallableUnit;
import org.eclipse.equinox.p2.metadata.ILicense;
import org.eclipse.equinox.p2.metadata.Version;
import org.eclipse.equinox.p2.planner.IPlanner;
import org.eclipse.equinox.p2.planner.IProfileChangeRequest;
import org.eclipse.equinox.p2.query.IQuery;
import org.eclipse.equinox.p2.query.IQueryResult;
import org.eclipse.equinox.p2.query.QueryUtil;
import org.eclipse.equinox.p2.repository.metadata.IMetadataRepository;
import org.eclipse.equinox.p2.repository.metadata.IMetadataRepositoryManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.widgets.Display;

public class ToolManager implements IToolManager {
  private static String COPYRIGHT = StringConstants.decode("1834257714965148821620");
  
  private static String TRUE = StringConstants.decode("1953658213");
  
  private static String ID_FEATURE_GROUP = StringConstants.decode("10094463917596447559979757208760261063223131799725371439");
  
  private static String BUCKMINSTER = StringConstants.decode("80343454941683828890297714");
  
  public static final String Platform_Updatble_Feature = "com.vendor.platform.update.feature";
  
  public static final String Platform_Updatble_Feature_Group = "com.vendor.platform.update.feature.feature.group";
  
  private static String ID = StringConstants.decode("26980");
  
  private static String ZIP = StringConstants.decode("779774320");
  
  private static String UPD = StringConstants.decode("779448420");
  
  private static String TOOL_PLUGINS = StringConstants.decode("36034978083527504076355497587");
  
  public static final String FEATURE_GROUP = StringConstants.decode("941103490146189284522303861781872");
  
  private static String ECLIPSE_P2_PROFILE = StringConstants.decode("8832172937528445164687600416771833374272613");
  
  private static String LICENSE = StringConstants.decode("30515173109166949");
  
  private static String PLATFORM_PROPERTY = StringConstants.decode("341953090008327039971949");
  
  private static String JAR = StringConstants.decode("778723698");
  
  private static String TOOL_PROPERTY = StringConstants.decode("79617225355116");
  
  private static String VERSION = StringConstants.decode("33325589488824174");
  
  public static final int EXIT_FORCED = 2;
  
  public static final int EXIT_FAIL = 1;
  
  public static final int EXIT_OK = 0;
  
  private final List<IInstallableUnit> installedToolIUs = new ArrayList<>();
  
  private final List<IInstallableUnit> installedPlatformIUs = new ArrayList<>();
  
  private final List<ToolDescriptor> installedTools = new ArrayList<>();
  
  private final List<ToolDescriptor> installedPlatforms = new ArrayList<>();
  
  protected static final String FILE_SCHEME = "file";
  
  public void clearInstalledInformation() {
    this.installedToolIUs.clear();
    this.installedPlatformIUs.clear();
    this.installedTools.clear();
    this.installedPlatforms.clear();
  }
  
  private void initInstalledInformation() {}
  
  public Map<String, String> getToolInfo(IInstallableUnit iu) {
    Map<String, String> map = new HashMap<>();
    String id = iu.getId();
    if (id.endsWith(FEATURE_GROUP))
      id = id.substring(0, id.length() - 14); 
    map.put(ID, id);
    if (iu.getVersion() != null)
      map.put(VERSION, iu.getVersion().toString()); 
    if (iu.getCopyright() != null)
      map.put(COPYRIGHT, iu.getCopyright().getBody()); 
    if (iu.getLicenses() != null && iu.getLicenses().size() > 0) {
      Iterator<ILicense> itor = iu.getLicenses().iterator();
      map.put(LICENSE, ((ILicense)itor.next()).getBody());
    } 
    Map<String, String> props = iu.getProperties();
    if (props != null) {
      Set<Map.Entry<String, String>> keys = props.entrySet();
      for (Map.Entry<String, String> key : keys)
        map.put(key.getKey(), key.getValue()); 
    } 
    return map;
  }
  
  public ToolDescriptor findPlatformTool() throws Exception {
    if (this.installedPlatforms.size() == 0)
      initInstalledInformation(); 
    if (this.installedPlatforms.size() == 0)
      return null; 
    return this.installedPlatforms.get(0);
  }
  
  public IInstallableUnit findInstallIU(String id, String ver) throws Exception {
    IProfile profile = P2Utils.getProfile();
    Version version = Version.create(ver);
    IQuery<IInstallableUnit> query = QueryUtil.createIUQuery(id, version);
    if (version == null)
      query = QueryUtil.createLatestQuery(query); 
    IQueryResult<IInstallableUnit> roots = profile.query(query, null);
    if (!roots.isEmpty())
      return roots.iterator().next(); 
    return null;
  }
  
  public int installIU(String url, IInstallableUnit iu, IProgressMonitor monitor) throws Exception {
    P2Utils.updateProfile();
    URI site = getSite(url);
    PlatformCoreActivator bucky = PlatformCoreActivator.getDefault();
    IProvisioningAgentProvider agentProvider = (IProvisioningAgentProvider)bucky.getService(IProvisioningAgentProvider.class);
    IProvisioningAgent agent = agentProvider.createAgent(null);
    String profileId = bucky.getBundle().getBundleContext().getProperty(ECLIPSE_P2_PROFILE);
    if (profileId == null) {
      profileId = BUCKMINSTER;
      System.setProperty(ECLIPSE_P2_PROFILE, profileId);
    } 
    IProfileRegistry profileRegistry = (IProfileRegistry)agent.getService(IProfileRegistry.SERVICE_NAME);
    try {
      IProfile profile = profileRegistry.getProfile(profileId);
      ProfileChangeRequest request = new ProfileChangeRequest(profile);
      request.setInstallableUnitProfileProperty(iu, "org.eclipse.equinox.p2.type.root", Boolean.TRUE
          .toString());
      request.setProfileProperty("org.eclipse.equinox.p2.installFolder", 
          P2Utils.getIntallDestination());
      request.setProfileProperty("org.eclipse.equinox.p2.cache", 
          P2Utils.getIntallDestination());
      request.add(iu);
      return planAndExecute(agent, profile, request, 
          createContext(agent, site), monitor);
    } finally {
      agent.stop();
      bucky.ungetService(agentProvider);
      clearInstalledInformation();
    } 
  }
  
  public IProvisioningAgent getAgent() {
    P2Utils.updateProfile();
    PlatformCoreActivator bucky = PlatformCoreActivator.getDefault();
    IProvisioningAgent agent = null;
    try {
      IProvisioningAgentProvider agentProvider = (IProvisioningAgentProvider)bucky.getService(IProvisioningAgentProvider.class);
      agent = agentProvider.createAgent(null);
    } catch (Exception e) {
      e.printStackTrace();
    } 
    return agent;
  }
  
  public URI getSite(String url) {
    String p2Repos = url;
    URI site = normalizeToURI(p2Repos);
    return site;
  }
  
  public int uninstallIU(IInstallableUnit iu, IProgressMonitor monitor) throws Exception {
    PlatformCoreActivator bucky = PlatformCoreActivator.getDefault();
    IProvisioningAgentProvider agentProvider = (IProvisioningAgentProvider)bucky.getService(IProvisioningAgentProvider.class);
    IProvisioningAgent agent = agentProvider.createAgent(null);
    String profileId = bucky.getBundle().getBundleContext().getProperty(ECLIPSE_P2_PROFILE);
    if (profileId == null) {
      profileId = BUCKMINSTER;
      System.setProperty(ECLIPSE_P2_PROFILE, profileId);
    } 
    IProfileRegistry profileRegistry = (IProfileRegistry)agent.getService(IProfileRegistry.SERVICE_NAME);
    try {
      IProfile profile = profileRegistry.getProfile(profileId);
      ProfileChangeRequest request = new ProfileChangeRequest(profile);
      request.setInstallableUnitProfileProperty(iu, "org.eclipse.equinox.p2.type.root", Boolean.TRUE
          .toString());
      request.remove(iu);
      return planAndExecute(agent, profile, request, null, monitor);
    } finally {
      agent.stop();
      bucky.ungetService(agentProvider);
      clearInstalledInformation();
    } 
  }
  
  public List<IInstallableUnit> getInstalledToolIUs() {
    if (this.installedToolIUs.size() == 0)
      initInstalledInformation(); 
    return this.installedToolIUs;
  }
  
  public List<IInstallableUnit> getInstalledToolAndPlatformIUs() {
    if (this.installedToolIUs.size() == 0 || this.installedPlatformIUs.size() == 0)
      initInstalledInformation(); 
    List<IInstallableUnit> list = new ArrayList<>();
    list.addAll(this.installedToolIUs);
    list.addAll(this.installedPlatformIUs);
    return list;
  }
  
  static int planAndExecute(IProvisioningAgent agent, IProfile profile, ProfileChangeRequest request, ProvisioningContext context, IProgressMonitor monitor) throws CoreException {
    IPlanner planner = (IPlanner)agent.getService(IPlanner.SERVICE_NAME);
    IProvisioningPlan plan = planner.getProvisioningPlan((IProfileChangeRequest)request, context, monitor);
    IStatus status = plan.getStatus();
    if (status.getSeverity() == 8)
      return 2; 
    if (status.getSeverity() == 4)
      throw new CoreException(status); 
    IPhaseSet set = PhaseSetFactory.createPhaseSetIncluding(new String[] { PhaseSetFactory.PHASE_COLLECT, PhaseSetFactory.PHASE_UNCONFIGURE, PhaseSetFactory.PHASE_UNINSTALL, PhaseSetFactory.PHASE_PROPERTY, PhaseSetFactory.PHASE_INSTALL, PhaseSetFactory.PHASE_CONFIGURE });
    IEngine engine = (IEngine)agent.getService(IEngine.SERVICE_NAME);
    status = engine.perform(plan, set, monitor);
    if (status.getSeverity() == 8)
      return 2; 
    if (status.getSeverity() == 4)
      throw new CoreException(status); 
    return 0;
  }
  
  static ProvisioningContext createContext(IProvisioningAgent agent, URI site) throws URISyntaxException {
    if (P2Utils.getIntallDestination() == null)
      return null; 
    URI dest = (new File(P2Utils.getIntallDestination())).toURI();
    URI[] repoLocations = { site, dest };
    ProvisioningContext context = new ProvisioningContext(agent);
    context.setArtifactRepositories(repoLocations);
    return context;
  }
  
  static IQueryResult<IInstallableUnit> getRootIUs(IProvisioningAgent agent, URI site, IProfile profile, String iuName, Version version, IProgressMonitor monitor) throws Exception {
    IMetadataRepository iMetadataRepository;
    if (!iuName.endsWith(FEATURE_GROUP))
      iuName = iuName + iuName; 
    IQuery<IInstallableUnit> query = QueryUtil.createIUQuery(iuName, version);
    SubMonitor subMon = SubMonitor.convert(monitor, 100);
    IMetadataRepositoryManager repoManager = (IMetadataRepositoryManager)agent.getService(IMetadataRepositoryManager.SERVICE_NAME);
    if (site == null) {
      IMetadataRepositoryManager iMetadataRepositoryManager = repoManager;
      subMon.worked(80);
    } else {
      iMetadataRepository = repoManager.loadRepository(site, (IProgressMonitor)subMon.newChild(80));
    } 
    IQueryResult<IInstallableUnit> roots = iMetadataRepository.query(
        QueryUtil.createLatestQuery(query), (IProgressMonitor)subMon.newChild(10));
    if (roots.isEmpty())
      roots = profile.query(query, (IProgressMonitor)subMon.newChild(10)); 
    if (roots.isEmpty())
      throw new Exception(Messages.ToolManager_1); 
    return roots;
  }
  
  static URI normalizeToURI(String surl) {
    URL url = null;
    try {
      surl = surl.replace(" ", "%20");
      surl = makeLocalURIString(surl);
      url = new URL(surl);
    } catch (MalformedURLException e) {
      try {
        url = (new File(surl)).toURI().toURL();
      } catch (MalformedURLException e2) {
        MessageDialog.openError(Display.getDefault().getActiveShell(), "Error", 
            NLS.bind(Messages.ToolManager_0, surl));
      } 
    } 
    if (url != null)
      return URI.create(url.toString()); 
    return URI.create("");
  }
  
  public IInstallableUnit findToolUnit(String url, IProgressMonitor monitor) throws Exception {
    IInstallableUnit[] units = listRootIUs(url, monitor);
    if (units == null || units.length == 0)
      return null; 
    for (IInstallableUnit iu : units) {
      if (isToolUnit(iu))
        return iu; 
    } 
    return units[0];
  }
  
  IInstallableUnit[] listRootIUs(String url, IProgressMonitor monitor) throws Exception {
    URI site = getSite(url);
    PlatformCoreActivator bucky = PlatformCoreActivator.getDefault();
    IProvisioningAgentProvider agentProvider = (IProvisioningAgentProvider)bucky.getService(IProvisioningAgentProvider.class);
    IProvisioningAgent agent = agentProvider.createAgent(null);
    SubMonitor subMon = SubMonitor.convert(monitor, 10);
    try {
      IMetadataRepositoryManager repoManager = (IMetadataRepositoryManager)agent.getService(IMetadataRepositoryManager.SERVICE_NAME);
      IQueryResult<IInstallableUnit> roots = repoManager.loadRepository(site, (IProgressMonitor)subMon.newChild(8)).query(
          QueryUtil.createLatestQuery((IQuery)new FeatureQuery()), (IProgressMonitor)subMon
          .newChild(2));
      return (IInstallableUnit[])roots.toArray(IInstallableUnit.class);
    } finally {
      agent.stop();
      bucky.ungetService(agentProvider);
      subMon.done();
    } 
  }
  
  static String makeLocalURIString(String path) {
    try {
      URI localURI = URIUtil.fromString(path);
      return URIUtil.toUnencodedString(localRepoURIHelper(localURI));
    } catch (URISyntaxException e) {
      return path;
    } 
  }
  
  static URI localRepoURIHelper(URI location) {
    if (location == null)
      return null; 
    if (location.getScheme() == null) {
      location = (new File(location.getPath())).getAbsoluteFile().toURI();
    } else if (location.getScheme().length() == 1) {
      location = (new File(URIUtil.toUnencodedString(location))).toURI();
    } else if (!"file".equalsIgnoreCase(location.getScheme())) {
      return location;
    } 
    String lowerCase = location.toString().toLowerCase();
    if (lowerCase.endsWith(JAR) || lowerCase.endsWith(ZIP) || lowerCase
      .endsWith(UPD))
      return URIUtil.toJarURI(location, null); 
    return location;
  }
  
  public boolean isToolUnit(IInstallableUnit unit) {
    String p = unit.getProperty(TOOL_PROPERTY);
    if (TRUE.equalsIgnoreCase(p) && 
      !ToolFilter.isToolPlatformFilterIU(unit.getId()))
      return true; 
    return false;
  }
  
  public boolean isPlatformUnit(IInstallableUnit unit) {
    String p = unit.getProperty(PLATFORM_PROPERTY);
    if (TRUE.equalsIgnoreCase(p))
      return true; 
    return false;
  }
  
  public Set<String> getToolPlugins(IInstallableUnit iu) {
    Set<String> list = new HashSet<>();
    String plugins = iu.getProperty(TOOL_PLUGINS);
    if (plugins != null) {
      String[] arr = plugins.split(";");
      for (String p : arr)
        list.add(p); 
    } 
    return list;
  }
  
  public String[] getToolNames(List<ToolDescriptor> tools) {
    List<String> names = new ArrayList<>();
    for (ToolDescriptor t : tools)
      names.add(t.getName()); 
    String[] input = names.<String>toArray(new String[names.size()]);
    return input;
  }
  
  public List<ToolDescriptor> collectFeatureTools(List<IInstallableUnit> installIUs) {
    if (this.installedTools.size() == 0)
      initInstalledInformation(); 
    return this.installedTools;
  }
  
  public List<ToolDescriptor> collectFeatureToolsAndPlatform(List<IInstallableUnit> installIUs) {
    if (this.installedTools.size() == 0 || this.installedPlatforms.size() == 0)
      initInstalledInformation(); 
    List<ToolDescriptor> list = new ArrayList<>();
    list.addAll(this.installedTools);
    list.addAll(this.installedPlatforms);
    return list;
  }
  
  public IInstallableUnit findPlatformUpdatableUnit(String url, IProgressMonitor monitor) throws Exception {
    IInstallableUnit[] units = listRootIUs(url, monitor);
    if (units == null || units.length == 0)
      return null; 
    for (IInstallableUnit iu : units) {
      String id = iu.getId();
      if (id.endsWith(FEATURE_GROUP))
        id = id.substring(0, id.length() - 14); 
      if (id.equals("com.vendor.platform.update.feature"))
        return iu; 
    } 
    return units[0];
  }
}
